import logging
from urllib.parse import urljoin

import requests
from requests.exceptions import RequestException
from . import __title__, constants

logger = logging.getLogger(__name__)


class WebClient:
    """Client for using the Web API of the Discord Bridge server"""

    def __init__(
        self,
        host: str = constants.DEFAULT_HOST,
        port: int = constants.DEFAULT_PORT,
        timeout=constants.DEFAULT_CLIENT_TIMEOUT,
        disable_ping=False,
    ) -> None:
        """Connects to the server

        Args:
            host: host address the bridge server is running on
            port: port the bridge server is running on
            timeout: connection timeout in seconds
            disable_ping: set True to disable initial server ping

        Exceptions:
            OSError: in case the server is not up
        """
        self._base_url = f"http://{str(host)}:{int(port)}"
        self._timeout = timeout
        if not disable_ping and not self.ping():
            raise OSError(f"Server not reachable at {self._base_url}")

    def _build_url(self, route: str = "") -> str:
        return urljoin(self._base_url, str(route))

    def ping(self) -> bool:
        """Checks if the server is up

        Returns:
            True if the server is up, else False
        """
        try:
            r = requests.get(self._build_url(), timeout=self._timeout)
        except RequestException:
            return False
        return True if r.ok and __title__ in r.text else False

    def send_channel_message(
        self, channel_id: int, content: str = None, embed: dict = None
    ) -> None:
        """posts a message in the given channel

        Args:
            channel_id: Discord ID of the channel
            content: message text
            embed: embed to add to the message (see Discord API documentation for correct format)

        Exceptions:
            requests.exceptions.HTTPError: 404: Channel not found

        """
        params = {
            "channel_id": int(channel_id),
        }
        if content:
            params["content"] = str(content)
        if embed:
            params["embed"] = embed
        r = requests.post(
            self._build_url("send_channel_message"),
            json=params,
            timeout=self._timeout,
        )
        r.raise_for_status()

    def send_direct_message(
        self, user_id: int, content: str = None, embed: dict = None
    ):
        """posts a direct message to a user

        Args:
            user_id: Discord ID of the user
            content: message text
            embed: embed to add to the message (see Discord API documentation for correct format)

        Exceptions:
            requests.exceptions.HTTPError: 404: User not found

        """
        params = {
            "user_id": int(user_id),
        }
        if content:
            params["content"] = str(content)
        if embed:
            params["embed"] = embed
        r = requests.post(
            self._build_url("send_direct_message"),
            json=params,
            timeout=self._timeout,
        )
        r.raise_for_status()



import json
import unittest
from urllib.parse import urljoin

import requests_mock
from requests.exceptions import ConnectionError, ConnectTimeout

from .. import __title__, constants
from ..client import WebClient


@requests_mock.Mocker()
class WebClientTestCase(unittest.TestCase):
    def setUp(self) -> None:
        self._base_url = f"http://{constants.DEFAULT_HOST}:{constants.DEFAULT_PORT}"

    def _build_url(self, route: str = "") -> str:
        return urljoin(self._base_url, route)

    def test_should_create_client_when_server_available(self, requests_mocker):
        # given
        requests_mocker.register_uri(
            "GET", self._build_url(), status_code=200, text=__title__
        )
        # when
        client = WebClient()
        # then
        self.assertIsInstance(client, WebClient)

    def test_should_raise_OSError_when_server_not_available(self, requests_mocker):
        # given
        requests_mocker.register_uri("GET", self._build_url(), status_code=500)
        # when/then
        with self.assertRaises(OSError):
            WebClient()

    def test_should_create_client_when_server_not_available_and_ping_disabled(
        self, requests_mocker
    ):
        # given
        requests_mocker.register_uri(
            "GET", self._build_url(), status_code=200, text=__title__
        )
        # when
        client = WebClient(disable_ping=True)
        # then
        self.assertIsInstance(client, WebClient)

    def test_should_return_true_if_server_is_up(self, requests_mocker):
        # given
        requests_mocker.register_uri(
            "GET", self._build_url(), status_code=200, text=__title__
        )
        client = WebClient(disable_ping=True)
        # when
        result = client.ping()
        # then
        self.assertTrue(result)

    def test_should_return_false_if_wrong_server(self, requests_mocker):
        requests_mocker.register_uri(
            "GET", self._build_url(), status_code=200, text="wrong server"
        )
        client = WebClient(disable_ping=True)
        # when
        result = client.ping()
        # then
        self.assertFalse(result)

    def test_should_return_false_if_server_is_down_1(self, requests_mocker):
        requests_mocker.register_uri("GET", self._build_url(), status_code=500)
        client = WebClient(disable_ping=True)
        # when
        result = client.ping()
        # then
        self.assertFalse(result)

    def test_should_return_false_if_server_is_down_2(self, requests_mocker):
        requests_mocker.register_uri("GET", self._build_url(), exc=ConnectTimeout)
        client = WebClient(disable_ping=True)
        # when
        result = client.ping()
        # then
        self.assertFalse(result)

    def test_should_return_false_if_server_is_down_3(self, requests_mocker):
        requests_mocker.register_uri("GET", self._build_url(), exc=ConnectionError)
        client = WebClient(disable_ping=True)
        # when
        result = client.ping()
        # then
        self.assertFalse(result)

    def test_should_send_channel_message_content_only(self, requests_mocker):
        # given
        requests_mocker.register_uri(
            "POST", self._build_url("send_channel_message"), status_code=201
        )
        client = WebClient(disable_ping=True)
        # when
        client.send_channel_message(channel_id=1234, content="my message text")
        # then
        data = json.loads(requests_mocker.last_request.text)
        self.assertDictEqual(data, {"channel_id": 1234, "content": "my message text"})

    def test_should_send_channel_message_embed_only(self, requests_mocker):
        # given
        requests_mocker.register_uri(
            "POST", self._build_url("send_channel_message"), status_code=201
        )
        client = WebClient(disable_ping=True)
        # when
        client.send_channel_message(channel_id=1234, embed={"description": "dummy"})
        # then
        data = json.loads(requests_mocker.last_request.text)
        self.assertDictEqual(
            data, {"channel_id": 1234, "embed": {"description": "dummy"}}
        )

    def test_should_send_channel_message_content_and_embed(self, requests_mocker):
        # given
        requests_mocker.register_uri(
            "POST", self._build_url("send_channel_message"), status_code=201
        )
        client = WebClient(disable_ping=True)
        # when
        client.send_channel_message(
            channel_id=1234,
            content="my message text",
            embed={"description": "dummy"},
        )
        # then
        data = json.loads(requests_mocker.last_request.text)
        self.assertDictEqual(
            data,
            {
                "channel_id": 1234,
                "content": "my message text",
                "embed": {"description": "dummy"},
            },
        )

    def test_should_send_direct_message_contents_only(self, requests_mocker):
        # given
        requests_mocker.register_uri(
            "POST", self._build_url("send_direct_message"), status_code=201
        )
        client = WebClient(disable_ping=True)
        # when
        client.send_direct_message(user_id=42, content="my important text")
        # then
        data = json.loads(requests_mocker.last_request.text)
        self.assertDictEqual(data, {"user_id": 42, "content": "my important text"})

    def test_should_send_direct_message_embed_only(self, requests_mocker):
        # given
        requests_mocker.register_uri(
            "POST", self._build_url("send_direct_message"), status_code=201
        )
        client = WebClient(disable_ping=True)
        # when
        client.send_direct_message(user_id=42, embed={"description": "dummy"})
        # then
        data = json.loads(requests_mocker.last_request.text)
        self.assertDictEqual(data, {"user_id": 42, "embed": {"description": "dummy"}})

    def test_should_send_direct_message_contents_and_embed(self, requests_mocker):
        # given
        requests_mocker.register_uri(
            "POST", self._build_url("send_direct_message"), status_code=201
        )
        client = WebClient(disable_ping=True)
        # when
        client.send_direct_message(
            user_id=42,
            content="my important text",
            embed={"description": "dummy"},
        )
        # then
        data = json.loads(requests_mocker.last_request.text)
        self.assertDictEqual(
            data,
            {
                "user_id": 42,
                "content": "my important text",
                "embed": {"description": "dummy"},
            },
        )



from unittest.mock import Mock

from aiohttp.test_utils import AioHTTPTestCase, unittest_run_loop
from aiohttp import web
import discord
from discord.errors import NotFound, Forbidden

from ..views import routes


USERS = {1001: "user-1", 1002: "user-2", 1100: "forbidden user"}
USERS_FORBIDDEN = [1100]
CHANNELS = {2001: "channel-1", 2002: "channel-2", 2100: "forbidden channel"}
CHANNELS_FORBIDDEN = [2100]


class DiscordChannel:
    def __init__(self, id, name) -> None:
        self.id = id
        self.name = name

    async def send(self, content, embed=None):
        if content:
            assert isinstance(content, str)
        if embed:
            assert isinstance(embed, discord.Embed)
        if self.id in CHANNELS_FORBIDDEN:
            raise Forbidden(response=Mock(), message="Test:Forbidden channel")


class DiscordUser:
    def __init__(self, id, name) -> None:
        self.id = id
        self.name = name

    async def create_dm(self):
        if self.id in USERS_FORBIDDEN:
            return DiscordChannel(2100, "dm-2")
        else:
            return DiscordChannel(2101, "dm-1")


class DiscordStub:
    async def fetch_channel(self, channel_id):
        if channel_id in CHANNELS:
            if channel_id in CHANNELS_FORBIDDEN:
                raise Forbidden(response=Mock(), message="Test:Forbidden channel")
            return DiscordChannel(id=channel_id, name=CHANNELS[channel_id])
        raise NotFound(response=Mock(), message="Test:Unknown channel")

    async def fetch_user(self, user_id):
        if user_id in USERS:
            return DiscordUser(id=user_id, name=USERS[user_id])
        raise NotFound(response=Mock(), message="Test:Unknown user")


class TestOtherViews(AioHTTPTestCase):
    async def get_application(self):
        app = web.Application()
        app.add_routes(routes)
        app["discord_client"] = DiscordStub()
        return app

    @unittest_run_loop
    async def test_should_return_index(self):
        # when
        resp = await self.client.request("GET", "/")
        # then
        self.assertEqual(resp.status, 200)
        text = await resp.text()
        self.assertIn("Discord Bridge", text)


class TestPostChannelMessageView(AioHTTPTestCase):
    async def get_application(self):
        app = web.Application()
        app.add_routes(routes)
        app["discord_client"] = DiscordStub()
        return app

    @unittest_run_loop
    async def test_should_return_204_when_ok_content_only(self):
        # when
        resp = await self.client.request(
            "POST",
            "/send_channel_message",
            json={"channel_id": 2001, "content": "test_content"},
        )
        # then
        self.assertEqual(resp.status, 204)

    @unittest_run_loop
    async def test_should_return_204_when_ok_embed_only(self):
        # when
        resp = await self.client.request(
            "POST",
            "/send_channel_message",
            json={"channel_id": 2001, "embed": {"description": "dummy"}},
        )
        # then
        self.assertEqual(resp.status, 204)

    @unittest_run_loop
    async def test_should_return_204_when_ok_content_and_embed(self):
        # when
        resp = await self.client.request(
            "POST",
            "/send_channel_message",
            json={
                "channel_id": 2001,
                "content": "test_content",
                "embed": {"description": "dummy"},
            },
        )
        # then
        self.assertEqual(resp.status, 204)

    @unittest_run_loop
    async def test_should_return_400_when_mandatory_param_is_missing(self):
        # when
        resp = await self.client.request(
            "POST", "/send_channel_message", json={"content": "test_content"}
        )
        # then
        self.assertEqual(resp.status, 400)

    @unittest_run_loop
    async def test_should_return_400_when_both_content_and_embed_are_missing(self):
        # when
        resp = await self.client.request(
            "POST", "/send_channel_message", json={"channel_id": 2001}
        )
        # then
        self.assertEqual(resp.status, 400)

    @unittest_run_loop
    async def test_should_return_404_when_channel_is_unknown(self):
        # when
        resp = await self.client.request(
            "POST",
            "/send_channel_message",
            json={"channel_id": 666, "content": "test_content"},
        )
        # then
        self.assertEqual(resp.status, 404)

    @unittest_run_loop
    async def test_should_return_403_when_access_denied(self):
        # when
        resp = await self.client.request(
            "POST",
            "/send_channel_message",
            json={"channel_id": 2100, "content": "test_content"},
        )
        # then
        self.assertEqual(resp.status, 403)


class TestPostDirectMessageView(AioHTTPTestCase):
    async def get_application(self):
        app = web.Application()
        app.add_routes(routes)
        app["discord_client"] = DiscordStub()
        return app

    @unittest_run_loop
    async def test_should_return_204_when_ok_content_only(self):
        # when
        resp = await self.client.request(
            "POST",
            "/send_direct_message",
            json={"user_id": 1001, "content": "test_content"},
        )
        # then
        self.assertEqual(resp.status, 204)

    @unittest_run_loop
    async def test_should_return_204_when_ok_embed_only(self):
        # when
        resp = await self.client.request(
            "POST",
            "/send_direct_message",
            json={"user_id": 1001, "embed": {"description": "dummy"}},
        )
        # then
        self.assertEqual(resp.status, 204)

    @unittest_run_loop
    async def test_should_return_204_when_ok_content_and_embed(self):
        # when
        resp = await self.client.request(
            "POST",
            "/send_direct_message",
            json={
                "user_id": 1001,
                "content": "test_content",
                "embed": {"description": "dummy"},
            },
        )
        # then
        self.assertEqual(resp.status, 204)

    @unittest_run_loop
    async def test_should_return_400_when_mandatory_param_is_missing(self):
        # when
        resp = await self.client.request(
            "POST",
            "/send_direct_message",
            json={"content": "bla bla"},
        )
        # then
        self.assertEqual(resp.status, 400)

    @unittest_run_loop
    async def test_should_return_400_when_both_content_and_embed_are_missing(self):
        # when
        resp = await self.client.request(
            "POST", "/send_direct_message", json={"user_id": 1001}
        )
        # then
        self.assertEqual(resp.status, 400)

    @unittest_run_loop
    async def test_should_return_404_when_user_is_unknown(self):
        # when
        resp = await self.client.request(
            "POST",
            "/send_direct_message",
            json={"user_id": 666, "content": "test_content"},
        )
        # then
        self.assertEqual(resp.status, 404)

    @unittest_run_loop
    async def test_should_return_403_when_user_access_not_allowed(self):
        # when
        resp = await self.client.request(
            "POST",
            "/send_direct_message",
            json={"user_id": 1100, "content": "test_content"},
        )
        # then
        self.assertEqual(resp.status, 403)



import argparse
import logging
import logging.config
import os
from pathlib import Path
import sys

from aiohttp import web
import discord

from . import __title__, __version__, constants
from .views import routes


logger = logging.getLogger(__name__)
_LOG_FILE_NAME = "discordbridgesrv.log"


def setup_server(args_list: list) -> tuple:
    """parses all command line arguments and configures logging"""
    my_args = _parse_args(args_list)
    token = (
        os.environ.get("DISCORD_BOT_TOKEN") if my_args.token is None else my_args.token
    )
    if not token:
        print("ERROR: No Discord bot token provided")
        exit(1)
    logging.config.dictConfig(_logging_config(my_args))
    return token, my_args


def _parse_args(args_list: list) -> argparse.ArgumentParser:
    """returns parsed command line arguments"""
    my_arg_parser = argparse.ArgumentParser(
        description="Server with HTTP API for sending messages to Discord",
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
    )
    my_arg_parser.add_argument(
        "--token",
        help=(
            "Discord bot token. Can alternatively be specified as "
            "environment variable DISCORD_BOT_TOKEN."
        ),
    )
    my_arg_parser.add_argument(
        "--host", default=constants.DEFAULT_HOST, help="server host address"
    )
    my_arg_parser.add_argument(
        "--port", type=int, default=constants.DEFAULT_PORT, help="server port"
    )
    my_arg_parser.add_argument(
        "--log-level",
        default="INFO",
        help="Log level of log file",
        choices=["INFO", "WARN", "ERROR", "CRITICAL"],
    )
    my_arg_parser.add_argument(
        "--log-file-path",
        help=(
            "Path for storing the log file. If no path if provided, "
            "the log file will be stored in the current working folder"
        ),
    )
    my_arg_parser.add_argument(
        "--version",
        help="show the program version and exit",
        action="version",
        version=__version__,
    )
    return my_arg_parser.parse_args(args_list)


def _logging_config(my_args) -> dict:
    """returns dict to configure logging based on parsed args"""
    file_log_path_full = my_args.log_file_path
    filename = (
        Path(file_log_path_full) / _LOG_FILE_NAME
        if file_log_path_full
        else Path.cwd() / _LOG_FILE_NAME
    )
    print(f"Writing logfile to: {filename}")
    return {
        "version": 1,
        "disable_existing_loggers": False,
        "formatters": {
            "console": {"format": "[%(levelname)s] %(message)s"},
            "file": {"format": "%(asctime)s [%(levelname)s] %(name)s: %(message)s"},
        },
        "handlers": {
            "console": {
                "level": "CRITICAL",
                "formatter": "console",
                "class": "logging.StreamHandler",
                "stream": "ext://sys.stdout",  # Default is stderr
            },
            "file": {
                "level": my_args.log_level,
                "formatter": "file",
                "class": "logging.FileHandler",
                "filename": str(filename),
                "mode": "a",
            },
        },
        "loggers": {
            "": {  # root logger
                "handlers": ["console", "file"],
                "level": "DEBUG",
                "propagate": False,
            },
        },
    }


class MyClient(discord.Client):
    """Custom sub-class for handling Discord events"""

    async def on_connect(self):
        logger.info("%s has connected to Discord", self.user.name)

    async def on_ready(self):
        logger.info("%s as logged in successfully", self.user.name)

    async def on_disconnect(self):
        logger.info("%s has disconnected from Discord", self.user.name)


def run_server(token, my_args):
    """runs the server with given configuration until terminated"""

    async def on_shutdown(app):
        logger.info("Logging out from Discord...")
        await client.logout()

    logger.info(f"Server version: {__version__}")
    try:
        app = web.Application()
        app.add_routes(routes)
        app.on_shutdown.append(on_shutdown)
        discord.VoiceClient.warn_nacl = False
        client = MyClient()
        app["discord_client"] = client
        client.loop.create_task(client.start(token))
        logger.info("Starting server on http://%s:%s ...", my_args.host, my_args.port)
        web.run_app(app, host=my_args.host, port=my_args.port)
    finally:
        logger.info("Server has shut down")
        client.loop.close()


def main():
    """main function for starting the server"""
    print(f"{__title__} v{__version__}")
    print()
    run_server(*setup_server(sys.argv[1:]))


if __name__ == "__main__":
    main()



import os
from setuptools import find_packages, setup
from discordbridge import __version__

# read the contents of your README file
this_directory = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(this_directory, "README.md"), encoding="utf-8") as f:
    long_description = f.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name="discord-bridge",
    version=__version__,
    packages=find_packages(),
    include_package_data=True,
    license="MIT",
    description="HTTP bridge for the Discord API",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://gitlab.com/ErikKalkoken/discord-bridge",
    author="Erik Kalkoken",
    author_email="kalkoken87@gmail.com",
    classifiers=[
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ],
    python_requires="~=3.6",
    install_requires=["discord.py", "requests"],
    entry_points={
        "console_scripts": [
            "discordbridgesrv=discordbridge.server:main",
        ],
    },
)



from pathlib import Path
import unittest
from unittest.mock import patch

from .. import server
from .. import constants


MODULE_PATH = "discordbridge.server"


@patch(MODULE_PATH + ".logging.config.dictConfig", lambda x: None)
class TestSetupServer(unittest.TestCase):
    def test_should_return_token_when_provided_in_args(self):
        # when
        token, my_args = server.setup_server(["--token", "abc"])
        # then
        self.assertEqual(token, "abc")

    @patch(MODULE_PATH + ".os.environ")
    def test_should_return_token_when_provided_as_environ_var(self, mock_environ):
        # given
        mock_environ.get.return_value = "abc"
        # when
        token, my_args = server.setup_server([])
        # then
        self.assertEqual(token, "abc")

    def test_should_have_all_defaults(self):
        # when
        token, my_args = server.setup_server(["--token", "abc"])
        # then
        self.assertEqual(my_args.token, "abc")
        self.assertEqual(my_args.host, constants.DEFAULT_HOST)
        self.assertEqual(my_args.port, constants.DEFAULT_PORT)
        self.assertEqual(my_args.log_level, "INFO")
        self.assertIsNone(my_args.log_file_path)

    def test_should_set_log_file_path_to_cwd(self):
        # given
        my_args = server._parse_args([])
        # when
        logger_dict = server._logging_config(my_args)
        # then
        self.assertEqual(
            logger_dict["handlers"]["file"]["filename"],
            str(Path.cwd() / server._LOG_FILE_NAME),
        )

    def test_should_set_log_file_path_to_given_path(self):
        # given
        my_path = Path(__file__).parent
        my_args = server._parse_args(["--log-file-path", str(my_path)])
        # when
        logger_dict = server._logging_config(my_args)
        # then
        self.assertEqual(
            logger_dict["handlers"]["file"]["filename"],
            str(my_path / server._LOG_FILE_NAME),
        )
